
class StartEvent:
    """
    StartEvent documentation.
    """
    def __init__(self,
                 _id: str,
                 name: str) -> None:
        self._id = _id
        self.name = name

